/* generate source-package version-string C89-compatible include-file, from the
 * MJS-style "dist.inf" package-spec file.
 *
 * This program written in portable ANSI Standard C X3.159-1989 (C89).
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <locale.h>

static char *me = "?";
static char *symbol = NULL;
static char *pkgname = NULL;
static char *major = NULL;
static char *minor = NULL;
static char *micro = NULL;
static char *datestr = NULL;
static char *author = NULL;


static char *str_dup(char *s)
	{
	char *p;

	p = malloc(strlen(s) + 1);
	if (!p)
		{
		fprintf(stderr, "%s error: out of memory\n", me);
		exit(EXIT_FAILURE);
		}
	strcpy(p, s);
	return (p);
	}

static void opterror(const char *opt)
	{
	fprintf(stderr,"usage: %s: option \"%s\" expects an argument\n",
			me, opt);
	exit(EXIT_FAILURE);
	}

static char *getoval(int *ac, char ***av, const char *opt)
	{
	int n;
	char *arg;

	assert(*ac && *av && **av && ***av && opt && *opt);
	n = strlen(opt);
	if (!strncmp(**av, opt, n))
		{
		char *p = &(**av)[n];
		if (!*p)	/* try following arg */
			{
			--(*ac), ++(*av);
			if (!**av)
				opterror(opt);
			arg = **av;
			}
		else
			arg = p;
		if (*p == '-')
			opterror(opt);
		--(*ac), ++(*av);
		return (arg);
		}
	return (NULL);
	}

static int parse(char *src, char **key, char **value)
	{
	char *p, *q;
	int quoted;

	assert(key && value && src);
	p = strchr(src, '=');
	if (!p)
		return (0);
	*p++ = '\0';
	*key = src;
	*value = p;
	quoted = 0;
	for (q = p; q && *q; ++q)
		{
		if ((*q == '\r') || (*q == '\n'))
			break;
		if (*q == '#') 
			break;
		else if ((*q == '"') || (*q == '\''))
			quoted = !quoted;
		else if (!quoted && ((*q == ' ') || (*q == '\t')))
			break;
		else
			*p++ = *q;
		}
	*p = '\0';

	if (!strlen(*key) || !strlen(*value))
		return (0);
	return (1);
	}

static void read_input(const char *filename)
	{
	size_t n;
	char buf[1024], *key, *value;
	FILE *pf;

	pf = fopen(filename, "r");
	if (!pf)
		{
		fprintf(stderr, "%s error: cannot open file: %s\n",
			me, filename);
		exit(EXIT_FAILURE);
		}
	while (fgets(buf, sizeof(buf), pf))
		{
		n = strlen(buf);
		if (!n || (buf[n-1] != '\n'))
			{
			fprintf(stderr, "%s error: input line to long in %s\n",
				me, filename);
			exit(EXIT_SUCCESS);
			}
		if (parse(buf, &key, &value))
			{
			if (!strcmp(key, "pkgname"))
				pkgname = str_dup(value);
			else if (!strcmp(key, "major"))
				major = str_dup(value);
			else if (!strcmp(key, "minor"))
				minor = str_dup(value);
			else if (!strcmp(key, "micro"))
				micro = str_dup(value);
			else if (!strcmp(key, "dated"))
				datestr = str_dup(value);
			else if (!strcmp(key, "author"))
				author = str_dup(value);
			}
		}
	if (ferror(pf))
		{
		fprintf(stderr, "%s error: input failed on %s\n",
			me, filename);
		exit(EXIT_SUCCESS);
		}
	fclose(pf);

	if (pkgname && major && minor && datestr && author)
		return;
	fprintf(stderr, "%s error: incomplete input file: %s\n", me, filename);
	exit(EXIT_SUCCESS);
	}

static void write_output(const char *filename)
	{
	FILE *pf;

	pf = fopen(filename, "w");
	if (!pf)
		{
		fprintf(stderr, "%s error: cannot create file: %s\n",
			me, filename);
		exit(EXIT_FAILURE);
		}

	fprintf(pf, "%s\n%s\n%s\n",
		"/* computed source-package-version string.",
 		" * THIS FILE IS GENERATED AUTOMATICALLY - DO NOT EDIT",
		" */");
	fprintf(pf, "const char * const %s = \"@(#)%s %s.%s%s%s %s %s\";\n",
			symbol, pkgname,
			major, minor,
			(micro && strlen(micro)) ? "." : "",
			(micro && strlen(micro)) ? micro : "",
			datestr, author);
	if (ferror(pf))
		{
		fprintf(stderr, "%s error: write failed\n", me);
		exit(EXIT_FAILURE);
		}
	fclose(pf);
	}

static int check_symbolname(void)
	{
	char *p;

	if (!symbol || !isalpha(*symbol))
		return (0);
	for (p = symbol; p && *p; ++p)
		{
		if (!isalpha(*p) && !isdigit(*p) && (*p != '_'))
			return (0);
		}
	return (1);
	} 
			
int main(int ac, char **av)
	{
	char *outfile = NULL, *s;

	me = *av;
	--ac, ++av;

	setlocale(LC_CTYPE, "C");	/* paranoia */

	while (ac && *av)
		{
		if (!strcmp(*av, "--"))		/* explicit end-of-options */
			{
			--ac, ++av;
			break;
			}
		else if ((s = getoval(&ac, &av, "-s")) != NULL)
			symbol = s;
		else if ((s = getoval(&ac, &av, "-o")) != NULL)
			outfile = s;
		else if (**av == '-')	/* unrecognised option */
			{
			fprintf(stderr, "usage: %s -s* -o* input-file\n", me);
			exit(EXIT_FAILURE);
			}
		else		/* implicit end-of-options */
			break;
		}

	if (!symbol)
		{
		fprintf(stderr, "usage: %s: missing -s option (symbol-name)\n",
			me);
		exit(EXIT_FAILURE);
		}
	if (!check_symbolname())
		{
		fprintf(stderr, "%s error: invalid symbol name: %s\n",
			me, symbol);
		exit(EXIT_FAILURE);
		}

	if (!outfile)
		{
		fprintf(stderr, "usage: %s: missing -o option (output-file)\n",
			me);
		exit(EXIT_FAILURE);
		}
			
	if ((ac < 1) || !outfile)
		{
		fprintf(stderr, "usage: %s -s* -o* input-file\n", me);
		exit(EXIT_FAILURE);
		}

	read_input(*av);
	write_output(outfile);
	return (EXIT_SUCCESS);
	}
