/* MJS Portable C Library.
 * Module: low-level notification-message preformatting+emission
 * @(#) txtmsg.c 1.0 26dec2016
 *
 * Copyright (c) 2016 Mike Spooner
 *----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 * Source-code conforms to ANSI standard X3.159-1989, with optional
 * extensions to better support Win32 GUI-only applications.
 */
#include <string.h>
#include <stdlib.h>
#include "mjsuimpl.h"


/* forward declaration of default implementation: */
static VOID dflt_show(UINT type, const CHAR *pfx, const CHAR *fmt, va_list ap);

/* library private pointer to the active function used to actually display the
 * prepared messages:
 */
VOID (*mjs_notify)(UINT type,
	const CHAR *pfx, const CHAR *fmt, va_list ap) = dflt_show;

/* default implementation of display-a-notification-message.
 *
 * This default implementation emits the program-name as a caption (except
 * for "usage" messages), followed by the message-type prefix string,
 * the formatted message, and a newline, to standard-error.
 */
static VOID dflt_show(UINT type,
	const CHAR *pfx, const CHAR *fmt, va_list ap)
	{
	if (type != USAGE_NOTIFY)
		fprintf(stderr, "%s: ", process_name);
	fprintf(stderr, "%s", pfx);
	vfprintf(stderr, fmt, ap);
	fprintf(stderr, "\n");
	return;
	}

/* allow library-user to override or revert-to the default implementation:
 */
VOID notifier(VOID (*pfn)(UINT msgtype,
	const CHAR *pfx, const CHAR *fmt, va_list ap))
	{
	if (!pfn)
		mjs_notify = dflt_show;
	else
		mjs_notify = pfn;
	}
