/* MJS Portable C Library.
 * Module: additional error-message API - usage()
 * @(#) usage.c	1.1	26dec2016 MJS
 *
 * Copyright (c) 1991, 2016 Mike Spooner
 *----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 * Source-code conforms to ANSI standard X3.159-1989.
 */
#include <stdlib.h>
#include <string.h>
#include "mjsuimpl.h"


/* emit a fatal "usage" message.
 * Message is prefixed with "usage: " and the process name.
 */
VOID usage(CHAR *fmt, ...)
	{
	va_list ap;
	static const char *fallback_pfx = "usage error";
	static CHAR *pfx = NULL;	/* special prefix for usage messages */

	if (!pfx || (pfx == fallback_pfx))
		{
		const CHAR *s1 = "usage: ", *s3 = " ";
		pfx = malloc(strlen(s1) + strlen(process_name) +
				strlen(s3) + 1);
		if (!pfx)
			pfx = (CHAR *) fallback_pfx;
		else
			cpystr(pfx, s1, process_name, s3, NULL);
		}

	va_start(ap, fmt);
	(*mjs_notify)(USAGE_NOTIFY, pfx, fmt, ap);
	va_end(ap); 
	exit(EXIT_FAILURE);
	}

